from netbox.search import SearchIndex, register_search
from .models import Credential

class CredentialIndex(SearchIndex):
    model = Credential

@register_search
class CredentialIndex(SearchIndex):
    model = Credential
    fields = (
        ('service_type', 100),
        ('username', 1000),
        ('password', 5000),
        ('comments', 5000),
    )



from django.urls import path
from netbox.views.generic import ObjectChangeLogView

from . import models, views

urlpatterns = (
    path('credential/', views.CredentialListView.as_view(), name='credential_list'),
    path('credential/add/', views.CredentialEditView.as_view(), name='credential_add'),
    path('credential/import/', views.CredentialBulkImportView.as_view(), name='credential_import'),
    path('credential/<int:pk>/', views.CredentialView.as_view(), name='credential'),
    path('credential/<int:pk>/edit/', views.CredentialEditView.as_view(), name='credential_edit'),
    path('credential/<int:pk>/delete/', views.CredentialDeleteView.as_view(), name='credential_delete'),

    path('credential/delete/', views.CredentialBulkDeleteView.as_view(), name='credential_bulk_delete'),

    path('credential/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='credential_changelog', kwargs={
        'model': models.Credential
    }),

)



import django_tables2 as tables
from django.utils.translation import gettext_lazy as _

from netbox.tables import columns, NetBoxTable

from .models import Credential


class CredentialTable(NetBoxTable):
    class Meta(NetBoxTable.Meta):
        model = Credential
        fields = ('pk', 'id', 'service_type', 'parent', 'username', 'password', 'comments', 'actions')
        default_columns = ('service_type', 'parent', 'username', 'password')

    service_type = tables.Column(
        verbose_name=_('Service Type'),
        linkify=True
    )

    parent = tables.Column(
        verbose_name=_('Parent'),
        linkify=True,
        order_by=('device', 'virtual_machine')
    )




import django_filters
from django.db.models import Q
from django.utils.translation import gettext as _

from dcim.models import Device, Interface, Region, Site, SiteGroup
from netbox.filtersets import ChangeLoggedModelFilterSet, OrganizationalModelFilterSet, NetBoxModelFilterSet
from virtualization.models import VirtualMachine, VMInterface
from .models import *


class CredentialFilterSet(NetBoxModelFilterSet):
    device_id = django_filters.ModelMultipleChoiceFilter(
        queryset=Device.objects.all(),
        label=_('Device (ID)'),
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label=_('Device (name)'),
    )
    virtual_machine_id = django_filters.ModelMultipleChoiceFilter(
        queryset=VirtualMachine.objects.all(),
        label=_('Virtual machine (ID)'),
    )
    virtual_machine = django_filters.ModelMultipleChoiceFilter(
        field_name='virtual_machine__name',
        queryset=VirtualMachine.objects.all(),
        to_field_name='name',
        label=_('Virtual machine (name)'),
    )

    class Meta:
        model = Credential
        fields = ['id', 'service_type', 'username']

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(service_type__icontains=value) | Q(username__icontains=value)
        return queryset.filter(qs_filter)



from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError
from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel


class Credential(NetBoxModel):
    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.CASCADE,
        related_name='credentials',
        verbose_name=_('device'),
        null=True,
        blank=True
    )
    virtual_machine = models.ForeignKey(
        to='virtualization.VirtualMachine',
        on_delete=models.CASCADE,
        related_name='credentials',
        null=True,
        blank=True
    )
    # e.g. Telnet, SSH, HTTP, RDP, VNC, ...
    service_type = models.CharField(max_length=100)
    username = models.CharField(max_length=100)
    password = models.CharField(max_length=100, blank=True)
    comments = models.TextField(blank=True)

    def __str__(self):
        return self.username

    def get_absolute_url(self):
        return reverse('plugins:netbox_cybex:credential', args=[self.pk])

    @property
    def parent(self):
        return self.device or self.virtual_machine

    def clean(self):
        super().clean()

        # A Service must belong to a Device *or* to a VirtualMachine
        if self.device and self.virtual_machine:
            raise ValidationError(_("A credential cannot be associated with both a device and a virtual machine."))
        if not self.device and not self.virtual_machine:
            raise ValidationError(_("A credential must be associated with either a device or a virtual machine."))



from django.utils.translation import gettext_lazy as _
from utilities.forms.fields import CommentField, DynamicModelChoiceField, CSVModelChoiceField, DynamicModelMultipleChoiceField
from netbox.forms import NetBoxModelForm, NetBoxModelImportForm, NetBoxModelFilterSetForm
from dcim.models import Device
from virtualization.models import VirtualMachine

from .models import Credential


class CredentialFilterForm(NetBoxModelFilterSetForm):
    model = Credential
    fieldsets = (
        (None, ('q', 'filter_id')),
        (_('Assignment'), ('device_id', 'virtual_machine_id')),
    )
    device_id = DynamicModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False,
        label=_('Device'),
    )
    virtual_machine_id = DynamicModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False,
        label=_('Virtual Machine'),
    )


class CredentialForm(NetBoxModelForm):
    class Meta:
        model = Credential
        fields = ('service_type', 'device', 'virtual_machine', 'username', 'password', 'comments', 'tags')

    device = DynamicModelChoiceField(
        label=_('Device'),
        queryset=Device.objects.all(),
        required=False,
        selector=True
    )
    virtual_machine = DynamicModelChoiceField(
        label=_('Virtual machine'),
        queryset=VirtualMachine.objects.all(),
        required=False,
        selector=True
    )
    comments = CommentField()


class CredentialImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        label=_('Device'),
        queryset=Device.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Required if not assigned to a VM')
    )
    virtual_machine = CSVModelChoiceField(
        label=_('Virtual machine'),
        queryset=VirtualMachine.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Required if not assigned to a device')
    )

    class Meta:
        model = Credential
        fields = (
            'service_type', 'device', 'virtual_machine', 'username', 'password', 'comments', 'tags',
        )



from netbox.api.viewsets import NetBoxModelViewSet

from .. import filtersets, models
from .serializers import CredentialSerializer


class CredentialViewSet(NetBoxModelViewSet):
    queryset = models.Credential.objects.prefetch_related('tags')
    serializer_class = CredentialSerializer



from extras.plugins import PluginConfig

class NetBoxCybExConfig(PluginConfig):
    name = 'netbox_cybex'
    verbose_name = ' NetBox CybEx'
    description = 'Features for cyber exercises in NetBox'
    version = '0.1'
    base_url = 'cybex'
    min_version = '3.4.0'

config = NetBoxCybExConfig



from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ..models import Credential

class CredentialSerializer(NetBoxModelSerializer):

    class Meta:
        model = Credential
        fields = (
            'id', 'display', 'virtual_machine', 'service_type', 'username', 'password', 'comments',
            'tags', 'custom_fields', 'created', 'last_updated',
        )



# Generated by Django 4.2.9 on 2024-02-14 15:44

from django.db import migrations, models
import django.db.models.deletion
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0185_gfk_indexes'),
        ('extras', '0106_bookmark_user_cascade_deletion'),
        ('virtualization', '0038_virtualdisk'),
    ]

    operations = [
        migrations.CreateModel(
            name='Credential',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('service_type', models.CharField(max_length=100)),
                ('username', models.CharField(max_length=100)),
                ('password', models.CharField(blank=True, max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('device', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='credentials', to='dcim.device')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('virtual_machine', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='credentials', to='virtualization.virtualmachine')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from graphene import ObjectType
from netbox.graphql.types import NetBoxObjectType
from netbox.graphql.fields import ObjectField, ObjectListField
from . import models


class CredentialType(NetBoxObjectType):

    class Meta:
        model = models.Credential
        fields = '__all__'


class Query(ObjectType):
    credential = ObjectField(CredentialType)
    credential_list = ObjectListField(CredentialType)


schema = Query



from django.utils.translation import gettext as _
from extras.plugins import PluginMenuItem, PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


credentiallist_buttons = [
    PluginMenuButton(
        link='plugins:netbox_cybex:credential_add',
        title=_('Add'),
        icon_class='mdi mdi-plus-thick',
        color=ButtonColorChoices.GREEN,
    ),
    PluginMenuButton(
        link='plugins:netbox_cybex:credential_import',
        title=_('Import'),
        icon_class='mdi mdi-upload',
        color=ButtonColorChoices.CYAN,
    ),
]

menu_items = (
    PluginMenuItem(
        link='plugins:netbox_cybex:credential_list',
        link_text=_('Credentials'),
        buttons=credentiallist_buttons,
    ),
)



from netbox.views import generic
from . import filtersets, forms, models, tables


class CredentialView(generic.ObjectView):
    queryset = models.Credential.objects.all()
    template_name = 'netbox_cybex/credential.html'


class CredentialListView(generic.ObjectListView):
    queryset = models.Credential.objects.all()
    table = tables.CredentialTable
    filterset = filtersets.CredentialFilterSet
    filterset_form = forms.CredentialFilterForm


class CredentialEditView(generic.ObjectEditView):
    queryset = models.Credential.objects.all()
    form = forms.CredentialForm
    template_name = 'netbox_cybex/credential_edit.html'

class CredentialDeleteView(generic.ObjectDeleteView):
    queryset = models.Credential.objects.all()


class CredentialBulkImportView(generic.BulkImportView):
    queryset = models.Credential.objects.all()
    model_form = forms.CredentialImportForm


class CredentialBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Credential.objects.all()
    filterset = filtersets.CredentialFilterSet
    table = tables.CredentialTable
